@@ -1,3 +1,4 @@
+from . import acquisition
 from . import hdf_utils
 from . import pixel_utils
 from . import analysis
@@ -9,6 +10,7 @@ from . import pixel
 from . import line
 
 __all__ = ['line', 'pixel']
+__all__ += acquisition.__all__
 __all__ += hdf_utils.__all__
 __all__ += pixel_utils.__all__
 __all__ += analysis.__all__
