@@ -38,7 +38,7 @@ class GKPFM(FFtrEFM):
     Example usage:
 
         >> from ffta.hdf_utils import process
-        >> data = process.FFtrEFM(h5_main)
+        >> data = process.GKPFM(h5_main)
         >> data.test([1,2]) # tests on pixel 1,2 in row, column
         >> data.compute()
         >> data.reshape() # reshapes the tFP, shift data
@@ -117,7 +117,10 @@ class GKPFM(FFtrEFM):
                          **kwargs)
 
         # save Transfer Function
-        self.process_tf(exc_floor)
+        if not any(TF_norm):
+            self.process_tf(exc_floor)
+        else:
+            self.TF_norm = TF_norm
         # defl = get_utils.get_pixel(self.h5_main, [0, 0], array_form=True).flatten()
         # _gk = GKPixel(defl, self.parm_dict, exc_wfm=exc_wfm)
         # _gk.load_tf(self.parm_dict['tip_response'], self.parm_dict['tip_excitation'])
@@ -163,7 +166,7 @@ class GKPFM(FFtrEFM):
         
         return
 
-    def test(self, pixel_ind=[0, 0], phases_to_test=[2.0708, 2.1208, 2.1708]):
+    def test(self, pixel_ind=[0, 0], phases_to_test=[2.0708, 2.1208, 2.1708], smooth=None):
         """
         Test the Pixel analysis of a single pixel
 
@@ -200,7 +203,9 @@ class GKPFM(FFtrEFM):
         _gk = GKPixel(defl, self.parm_dict, exc_wfm=self.exc_wfm,
                       TF_norm=self.TF_norm)
 
-        _gk.min_phase(phases_to_test=phases_to_test, noise_tolerance=self.parm_dict['noise_tolerance'])
+        _gk.min_phase(phases_to_test=phases_to_test, noise_tolerance=self.parm_dict['noise_tolerance'],
+                      verbose=False)
+        print("Set self.parm_dict['phase_shift'] to desired value")
         
         # If User supplies a phase shift
         if 'phase_shift' in self.parm_dict:
@@ -208,6 +213,7 @@ class GKPFM(FFtrEFM):
                           phase_shift = self.parm_dict['phase_shift'])
         else:
             _gk.force_out(plot=True, noise_tolerance=self.parm_dict['noise_tolerance'])
+            self.parm_dict['phase_shift'] = phases_to_test[-1]
 
         if self.parm_dict['denoise']:
             print('aa')
@@ -223,7 +229,7 @@ class GKPFM(FFtrEFM):
             print('bb')
             _gk.CPD = gaussian_filter1d(_gk.CPD, 1)[:_gk.num_CPD]
 
-        _gk.plot_cpd()
+        _gk.plot_cpd(smooth)
 
         self.cpd_dict = _gk._calc_cpd_params(return_dict=True, periods=self.parm_dict['periods'])
 
@@ -316,24 +322,9 @@ class GKPFM(FFtrEFM):
 
     def reshape(self):
         '''
-        Reshapes the tFP and shift data to be a matrix, then saves that dataset instead of the 1D
+        Currently unimplemented. See ffta.hdf_utils.process.FFtrEFM for an example
         '''
 
-        h5_cpd = self.h5_cpd[()]
-        h5_cap = self.h5_cap[()]
-
-        num_rows = self.parm_dict['num_rows']
-        num_cols = self.parm_dict['num_cols']
-
-        h5_cpd = np.reshape(h5_cpd, [num_rows, num_cols])
-        h5_cap = np.reshape(h5_cap, [num_rows, num_cols])
-
-        del self.h5_cpd.file[self.h5_cpd.name]
-        del self.h5_cpd.file[self.h5_cap.name]
-
-        self.h5_cpd = self.h5_results_grp.create_dataset('tfp', data=h5_cpd, dtype=np.float32)
-        self.h5_cap = self.h5_results_grp.create_dataset('shift', data=h5_cap, dtype=np.float32)
-
         return
 
     def _write_results_chunk(self):
@@ -397,7 +388,7 @@ class GKPFM(FFtrEFM):
         exc_wfm = args[2]
 
         gk = GKPixel(defl, parm_dict, exc_wfm=exc_wfm, TF_norm=TF_norm)
-        gk.force_out(noise_tolerance=parm_dict['noise_tolerance'])
+        gk.force_out(noise_tolerance=parm_dict['noise_tolerance'], phase_shift=parm_dict['phase_shift'])
 
         if parm_dict['denoise']:
             gk.noise_filter()
@@ -441,27 +432,22 @@ def save_CSV_from_file(h5_file, h5_path='/', append='', mirror=False):
         print('Saving from pyUSID object')
         h5_ff = h5_file.file
 
-    tfp = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'tfp')[0][()]
+    cpd = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'CPD')[0][()]
     # tfp_fixed = usid.hdf_utils.find_dataset(h5_file[h5_path], 'tfp_fixed')[0][()]
-    shift = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'shift')[0][()]
-
-    tfp_fixed, _ = badpixels.fix_array(tfp, threshold=2)
-    tfp_fixed = np.array(tfp_fixed)
-
-    print(usid.hdf_utils.find_dataset(h5_ff[h5_path], 'shift')[0].parent.name)
+    capacitance = usid.hdf_utils.find_dataset(h5_ff[h5_path], 'capacitance')[0][()]
+    
+    print(usid.hdf_utils.find_dataset(h5_ff[h5_path], 'CPD')[0].parent.name)
 
     path = h5_ff.file.filename.replace('\\', '/')
     path = '/'.join(path.split('/')[:-1]) + '/'
     os.chdir(path)
 
     if mirror:
-        np.savetxt('tfp-' + append + '.csv', np.fliplr(tfp).T, delimiter=',')
-        np.savetxt('shift-' + append + '.csv', np.fliplr(shift).T, delimiter=',')
-        np.savetxt('tfp_fixed-' + append + '.csv', np.fliplr(tfp_fixed).T, delimiter=',')
+        np.savetxt('cpd-' + append + '.csv', np.fliplr(cpd).T, delimiter=',')
+        np.savetxt('capacitance-' + append + '.csv', np.fliplr(capacitance).T, delimiter=',')
     else:
-        np.savetxt('tfp-' + append + '.csv', tfp.T, delimiter=',')
-        np.savetxt('shift-' + append + '.csv', shift.T, delimiter=',')
-        np.savetxt('tfp_fixed-' + append + '.csv', tfp_fixed.T, delimiter=',')
+        np.savetxt('cpd-' + append + '.csv', cpd.T, delimiter=',')
+        np.savetxt('capacitance-' + append + '.csv', capacitance.T, delimiter=',')
 
     return
 
