@@ -2,4 +2,4 @@ from . import analyze_h5
 from . import hdf_utils
 from . import process
 
-__all__ = ['hdf_utils', 'get_utils', 'analyze_h5', 'process']
\ No newline at end of file
+__all__ = ['hdf_utils', 'analyze_h5', 'process']
\ No newline at end of file
