@@ -37,8 +37,8 @@ class Pixel:
 
     signal_array : (n_points, n_signals) array_like
         2D real-valued signal array, corresponds to a pixel.
-    params : dict
-        Includes parameters for processing. The list of parameters is:
+    params : dict, optional
+        Includes parameters for processing, saved by the experiment Required:
 
         trigger = float (in seconds) (required)
         total_time = float (in seconds) (either this or sampling rate required)
@@ -78,12 +78,25 @@ class Pixel:
             hilbert: Hilbert transform method (default)
             wavelet: Morlet CWT approach
             stft: short time Fourier transform (sliding FFT)
+            fv: Feldman Force-Vib method
     
     filter_amplitude : bool, optional
         The Hilbert Transform amplitude can sometimes have drive frequency artifact.
     filter_frequency : bool, optional
         Filters the instantaneous frequency to remove noise peaks
-
+    recombination: bool, optional
+        Whether to invert the frequency (during a recombination or positive frequency shift event)
+    
+    The following are necessary to define a signal, either in params or explicitly
+    trigger: bool, optional
+        The point where the event occurs.
+    total_time: bool, optional
+        The total time of the signal
+    sampling_rate : bool, optional
+        The sampling rate. Note that sampling_rate * total_time must equal number of samples
+    roi : bool, opertional
+        The length of the window to find a minimum frequency peak
+        
     Attributes
     ----------
     n_points : int
@@ -96,6 +109,8 @@ class Pixel:
         Signal after phase-locking and averaging.
     tidx : int
         Index of trigger in time-domain.
+    amplitude : (n_points, ) array_like
+        Instantaneous amplitude of the signal
     phase : (n_points,) array_like
         Phase of the signal, only calculated with Hilbert Transform method.
     cwt_matrix : (n_widths, n_points) array_like
@@ -117,10 +132,6 @@ class Pixel:
     Frequency shift from wavelet analysis is not in Hertz. It should be used
     with caution.
 
-    analyze() does not raise an exception if there is one, however it logs the
-    exception if logging is turned on. This is implemented this way to avoid
-    crashing in the case of exception when this method is called from C API.
-
     References
     ----------
     .. \[1\] Giridharagopal R, Rayermann GE, Shao G, et al. Submicrosecond time
@@ -157,9 +168,11 @@ class Pixel:
                  fit_form='product', 
                  filter_amplitude=False,
                  filter_frequency=False,
+                 recombination=False,
                  trigger=None,
                  total_time=None,
-                 sampling_rate=None):
+                 sampling_rate=None,
+                 roi=None):
 
         # Create parameter attributes for optional parameters.
         # These defaults are overwritten by values in 'params'
@@ -179,8 +192,8 @@ class Pixel:
         self.fft_analysis = False
         self.fft_cycles = 2
         self.fft_params = {}  # for STFT
+        self.fft_time_res = 20e-6
 
-        self.recombination = False
         self.phase_fitting = False
         self.check_drive = True
         self.window = 'blackman'
@@ -191,6 +204,8 @@ class Pixel:
         self.fit_form = fit_form
         self.method = method
         self.filter_amplitude = filter_amplitude
+        self.filter_frequency = filter_frequency
+        self.recombination = recombination
 
         # Default Cantilever parameters, plugging in some reasonable defaults
         self.AMPINVOLS = 122e-9
@@ -206,11 +221,9 @@ class Pixel:
         self.signal_orig = None  # used in amplitude calc to undo any Windowing beforehand
 
         if len(signal_array.shape) == 2 and 1 not in signal_array.shape:
-
             self.n_points, self.n_signals = self.signal_array.shape
 
         else:
-
             self.n_signals = 1
             self.signal_array = self.signal_array.flatten()
             self.n_points = self.signal_array.shape[0]
@@ -220,17 +233,21 @@ class Pixel:
         if pycroscopy:
             self.signal_array = signal_array.T
         
-        setattr(self, 'trigger', trigger)
-        setattr(self, 'total_time', total_time)
-        setattr(self, 'sampling_rate', sampling_rate)
+        # The copy of the signal we will manipulate
+        self.signal = np.copy(self.signal_array)
         
         # Read parameter attributes from parameters dictionary.
         for key, value in params.items():
-            if not hasattr(self, key) or getattr(self, key) == None:
-                setattr(self, key, value)
+            setattr(self, key, value)
 
         for key, value in can_params.items():
             setattr(self, key, float(value))
+            
+        # Overwrite parameters with explicitly passed parameters
+        for key, val in zip(['trigger', 'total_time', 'sampling_rate', 'roi'], 
+                            [trigger, total_time, sampling_rate, roi]):
+            if val:
+                setattr(self, key, val)
 
         # Check for missing required parameters
         if self.trigger == None: # trigger can be 0
@@ -241,17 +258,28 @@ class Pixel:
                 raise KeyError('total_time or sampling_rate must be supplied')
             else:
                 self.total_time = self.sampling_rate * self.n_points
+        
         elif not self.sampling_rate:
             self.sampling_rate = int(self.n_points / self.total_time)
+        
         elif self.total_time != self.n_points / self.sampling_rate  :
             print(self.n_points / self.sampling_rate )
             print(self.total_time)
             raise ValueError('total_time and sampling_rate mismatch')
-        
-        if not hasattr(self, 'roi'):
+         
+        if self.total_time < self.trigger:
+            self.trigger = 0.1 * self.total_time 
+         
+        if not self.roi:
             self.roi = 0.3 * (self.total_time - self.trigger)
             warnings.warn('ROI defaulting to 30% post-trigger')
         
+        elif self.roi > self.total_time - self.trigger:
+            print(self.roi)
+            print(self.total_time - self.trigger)
+            warnings.warn('roi must not extend beyond the total_time; setting to maximum')
+            self.roi = self.total_time - self.trigger
+        
         self.tidx = int(self.trigger * self.sampling_rate)    
         self._tidx_orig = self.tidx
         self.tidx_orig = self.tidx
@@ -292,14 +320,41 @@ class Pixel:
 
         return
 
-    def remove_dc(self):
-        """Removes DC components from signals."""
+    def remove_dc(self, dc_width=10e3, plot=False):
+        """Removes DC components from each signal using FFT."""
 
-        if self.n_signals != 1:
-
-            for i in range(self.n_signals):
-                self.signal_array[:, i] -= np.mean(self.signal_array[:, i])
+        self.signal = np.copy(self.signal_array)
+        
+        if self.n_signals == 1:
+            
+            self.signal = np.reshape(self.signal, (self.n_points, self.n_signals))
 
+        for i in range(self.n_signals):
+            
+            f_ax = np.linspace(-self.sampling_rate/2, self.sampling_rate/2, self.n_points)
+            mid = int(len(f_ax) / 2)
+            # drive_bin = np.searchsorted(f_ax[mid:], self.drive_freq) + mid
+            delta_freq = self.sampling_rate / self.n_points
+    
+            SIG_DC = np.fft.fftshift(np.fft.fft(self.signal[:, i]))
+    
+            SIG_DC[:mid - int(dc_width / delta_freq)] = 0
+            SIG_DC[mid + int(dc_width / delta_freq):] = 0
+            sig_dc = np.real(np.fft.ifft(np.fft.ifftshift(SIG_DC)))
+    
+            self.signal[:, i] -= sig_dc
+        
+        if plot:
+            fig, ax = plt.subplots(nrows=2, figsize=(6, 10))
+            ax[0].plot(np.arange(0, self.total_time, 1/self.sampling_rate), sig_dc, 'b')
+            ax[1].plot(np.arange(0, self.total_time, 1/self.sampling_rate), self.signal_array, 'b')
+            ax[1].plot(np.arange(0, self.total_time, 1/self.sampling_rate), self.signal, 'r')
+            plt.title('DC Offset')
+            
+        if self.n_signals == 1:
+            
+            self.signal = self.signal[:,0]
+            
         return
 
     def phase_lock(self):
@@ -318,10 +373,9 @@ class Pixel:
         """Averages signals."""
 
         if self.n_signals != 1:  # if not multi-signal, don't average
-            self.signal = self.signal_array.mean(axis=1)
-
-        else:
-            self.signal = np.copy(self.signal_array)
+        
+            # self.signal = self.signal_array.mean(axis=1)
+            self.signal = self.signal.mean(axis=1)
 
         return
 
@@ -549,20 +603,24 @@ class Pixel:
         # Unwrap the phase.
         self.phase = np.unwrap(np.angle(self.signal))
 
-        if correct_slope:
-            # Remove the drive from phase.
-            # self.phase -= (2 * np.pi * self.drive_freq *
-            #               np.arange(self.n_points) / self.sampling_rate)
-
-            # A curve fit on the initial part to make sure that it worked.
-            start = int(0.3 * self.tidx)
-            end = int(0.7 * self.tidx)
-            fit = self.phase[start:end]
-
-            xfit = np.polyfit(np.arange(start, end), fit, 1)
-
-            # Remove the fit from phase.
-            self.phase -= (xfit[0] * np.arange(self.n_points)) + xfit[1]
+        try:
+            if correct_slope:
+                # Remove the drive from phase.
+                # self.phase -= (2 * np.pi * self.drive_freq *
+                #               np.arange(self.n_points) / self.sampling_rate)
+    
+                # A curve fit on the initial part to make sure that it worked.
+                start = int(0.3 * self.tidx)
+                end = int(0.7 * self.tidx)
+                fit = self.phase[start:end]
+    
+                xfit = np.polyfit(np.arange(start, end), fit, 1)
+    
+                # Remove the fit from phase.
+                self.phase -= (xfit[0] * np.arange(self.n_points)) + xfit[1]
+        except:
+            self.phase -= (2 * np.pi * self.drive_freq * 
+                           np.arange(self.n_points) / self.sampling_rate)
 
         self.phase = -self.phase  # need to correct for negative in DDHO solution
 
@@ -588,7 +646,162 @@ class Pixel:
         self.inst_freq = self.inst_freq_raw - self.inst_freq_raw[self.tidx]
 
         return
+    
+    def calculate_cwt(self, f_center=None, verbose=False, optimize=False, fit=False):
+        '''
+        Calculate instantaneous frequency using continuous wavelet transfer
+        
+        wavelet specified in self.wavelet. See PyWavelets CWT documentation
+        
+        Parameters
+        ----------
+        Optimize : bool, optionals
+            Currently placeholder for iteratively determining wavelet scales
+        '''
 
+        # wavlist = pywt.wavelist(kind='continuous')
+        # w0, wavelet_increment, cwt_scale = self.__get_cwt__()
+
+        # determine if scales will capture the relevant frequency
+        if not f_center:
+            f_center = self.drive_freq
+
+        dt = 1 / self.sampling_rate
+        sc = pywt.scale2frequency(self.wavelet, self.scales) / dt
+        
+        if self.verbose:
+            print('Wavelet scale from', np.min(sc), 'to', np.max(sc))
+
+        if f_center < np.min(sc) or f_center > np.max(sc):
+            raise ValueError('Choose a scale that captures frequency of interest')
+
+        if optimize:
+            print('!')
+            drive_bin = self.scales[np.searchsorted(sc, f_center)]
+            hi = int(1.2 * drive_bin)
+            lo = int(0.8 * drive_bin)
+            self.scales = np.arange(hi, lo, -0.1)
+
+        spectrogram, freq = pywt.cwt(self.signal, self.scales, self.wavelet, sampling_period=dt)
+
+        if not fit:
+
+            inst_freq, amplitude, _ = parab.ridge_finder(np.abs(spectrogram), np.arange(len(freq)))
+
+        # slow serial curve fitting
+        else:
+
+            inst_freq = np.zeros(self.n_points)
+            amplitude = np.zeros(self.n_points)
+
+            for c in range(spectrogram.shape[1]):
+
+                SIG = spectrogram[:, c]
+                if fit:
+                    pk = np.argmax(np.abs(SIG))
+                    popt = np.polyfit(np.arange(20),
+                                      np.abs(SIG[pk - 10:pk + 10]), 2)
+                    inst_freq[c] = -0.5 * popt[1] / popt[0]
+                    amplitude[c] = np.abs(SIG)[pk]
+
+        # rescale to correct frequency 
+        inst_freq = pywt.scale2frequency(self.wavelet, inst_freq + self.scales[0]) / dt
+
+        phase = spg.cumtrapz(inst_freq)
+        phase = np.append(phase, phase[-1])
+        tidx = int(self.tidx * len(inst_freq) / self.n_points)
+
+        self.amplitude = amplitude
+        self.inst_freq_raw = inst_freq
+        self.inst_freq = -1 * (inst_freq - inst_freq[tidx])  # -1 due to way scales are ordered
+        self.spectrogram = np.abs(spectrogram)
+        self.wavelet_freq = freq  # the wavelet frequencies
+
+        # subtract the w*t line (drive frequency line) from phase
+        start = int(0.3 * tidx)
+        end = int(0.7 * tidx)
+        xfit = np.polyfit(np.arange(start, end), phase[start:end], 1)
+        phase -= (xfit[0] * np.arange(len(inst_freq))) + xfit[1]
+
+        self.phase = phase
+
+        return
+
+    def calculate_stft(self, nfft=200):
+        '''
+        Sliding FFT approach
+        
+        Parameters
+        ----------
+        time_res : float, optional
+            What timescale to evaluate each FFT over
+                  
+        fit : bool, optional
+            Fits a parabola to the frequency peak to get the actual frequency
+            Otherwise defaults to parabolic interpolation (see parab.fit)
+        
+        nfft : int
+            Length of FFT calculated in the spectrogram. More points gets much slower
+            but the longer the FFT the finer the frequency bin spacing            
+        '''
+        
+        pts_per_ncycle = int(self.fft_time_res * self.sampling_rate)
+
+        if nfft < pts_per_ncycle:
+            print('Error with nfft setting')
+            nfft = pts_per_ncycle
+
+        if pts_per_ncycle > len(self.signal):
+            pts_per_ncycle = len(self.signal)
+
+        # drivebin = int(self.drive_freq / (self.sampling_rate / nfft ))
+        freq, times, spectrogram = sps.spectrogram(self.signal,
+                                                   self.sampling_rate,
+                                                   nperseg=pts_per_ncycle,
+                                                   noverlap=pts_per_ncycle - 1,
+                                                   nfft=nfft,
+                                                   window=self.window,
+                                                   mode='magnitude')
+
+        # Parabolic ridge finder
+        inst_freq, amplitude, _ = parab.ridge_finder(spectrogram, freq)
+
+        # Correctly pad the signals
+        _pts = self.n_points - len(inst_freq)
+        _pre = int(np.floor(_pts / 2))
+        _post = int(np.ceil(_pts / 2))
+
+        inst_freq = np.pad(inst_freq, (_pre, _post))
+        amplitude = np.pad(amplitude, (_pre, _post))
+
+        phase = spg.cumtrapz(inst_freq)
+        phase = np.append(phase, phase[-1])
+        tidx = int(self.tidx * len(inst_freq) / self.n_points)
+
+        self.amplitude = amplitude
+        self.inst_freq_raw = inst_freq
+        self.inst_freq = inst_freq - inst_freq[tidx]
+        self.spectrogram = spectrogram
+        self.stft_freq = freq
+        self.stft_times = times
+
+        # subtract the w*t line (drive frequency line) from phase
+        start = int(0.3 * tidx)
+        end = int(0.7 * tidx)
+        xfit = np.polyfit(np.arange(start, end), phase[start:end], 1)
+        phase -= (xfit[0] * np.arange(len(inst_freq))) + xfit[1]
+
+        self.phase = phase
+
+        return
+    
+    def forcevib(self):
+        """Calculate the resonance frequency directly via FORCE-VIB method.
+        See
+        """
+        
+        return
+    
     def find_tfp(self):
         """Calculate tfp and shift based self.fit_form and self.fit selection"""
         ridx = int(self.roi * self.sampling_rate)
@@ -672,151 +885,6 @@ class Pixel:
 
         return
 
-    def calculate_cwt(self, f_center=None, verbose=False, optimize=False, fit=False):
-        '''
-        Calculate instantaneous frequency using continuous wavelet transfer
-        
-        wavelet specified in self.wavelet. See PyWavelets CWT documentation
-        
-        Parameters
-        ----------
-        Optimize : bool, optionals
-            Currently placeholder for iteratively determining wavelet scales
-        '''
-
-        # wavlist = pywt.wavelist(kind='continuous')
-        # w0, wavelet_increment, cwt_scale = self.__get_cwt__()
-
-        # determine if scales will capture the relevant frequency
-        if not f_center:
-            f_center = self.drive_freq
-
-        dt = 1 / self.sampling_rate
-        sc = pywt.scale2frequency(self.wavelet, self.scales) / dt
-        
-        if self.verbose:
-            print('Wavelet scale from', np.min(sc), 'to', np.max(sc))
-
-        if f_center < np.min(sc) or f_center > np.max(sc):
-            raise ValueError('Choose a scale that captures frequency of interest')
-
-        if optimize:
-            print('!')
-            drive_bin = self.scales[np.searchsorted(sc, f_center)]
-            hi = int(1.2 * drive_bin)
-            lo = int(0.8 * drive_bin)
-            self.scales = np.arange(hi, lo, -0.1)
-
-        spectrogram, freq = pywt.cwt(self.signal, self.scales, self.wavelet, sampling_period=dt)
-
-        if not fit:
-
-            inst_freq, amplitude, _ = parab.ridge_finder(np.abs(spectrogram), np.arange(len(freq)))
-
-        # slow serial curve fitting
-        else:
-
-            inst_freq = np.zeros(self.n_points)
-            amplitude = np.zeros(self.n_points)
-
-            for c in range(spectrogram.shape[1]):
-
-                SIG = spectrogram[:, c]
-                if fit:
-                    pk = np.argmax(np.abs(SIG))
-                    popt = np.polyfit(np.arange(20),
-                                      np.abs(SIG[pk - 10:pk + 10]), 2)
-                    inst_freq[c] = -0.5 * popt[1] / popt[0]
-                    amplitude[c] = np.abs(SIG)[pk]
-
-        # rescale to correct frequency 
-        inst_freq = pywt.scale2frequency(self.wavelet, inst_freq + self.scales[0]) / dt
-
-        phase = spg.cumtrapz(inst_freq)
-        phase = np.append(phase, phase[-1])
-        tidx = int(self.tidx * len(inst_freq) / self.n_points)
-
-        self.amplitude = amplitude
-        self.inst_freq_raw = inst_freq
-        self.inst_freq = -1 * (inst_freq - inst_freq[tidx])  # -1 due to way scales are ordered
-        self.spectrogram = np.abs(spectrogram)
-        self.wavelet_freq = freq  # the wavelet frequencies
-
-        # subtract the w*t line (drive frequency line) from phase
-        start = int(0.3 * tidx)
-        end = int(0.7 * tidx)
-        xfit = np.polyfit(np.arange(start, end), phase[start:end], 1)
-        phase -= (xfit[0] * np.arange(len(inst_freq))) + xfit[1]
-
-        self.phase = phase
-
-        return
-
-    def calculate_stft(self, time_res=20e-6, nfft=200):
-        '''
-        Sliding FFT approach
-        
-        Parameters
-        ----------
-        time_res : float, optional
-            What timescale to evaluate each FFT over
-                  
-        fit : bool, optional
-            Fits a parabola to the frequency peak to get the actual frequency
-            Otherwise defaults to parabolic interpolation (see parab.fit)
-        
-        nfft : int
-            Length of FFT calculated in the spectrogram. More points gets much slower
-            but the longer the FFT the finer the frequency bin spacing            
-        '''
-
-        pts_per_ncycle = int(time_res * self.sampling_rate)
-
-        if nfft < pts_per_ncycle:
-            print('Error with nfft setting')
-            nfft = pts_per_ncycle
-
-        # drivebin = int(self.drive_freq / (self.sampling_rate / nfft ))
-        freq, times, spectrogram = sps.spectrogram(self.signal,
-                                                   self.sampling_rate,
-                                                   nperseg=pts_per_ncycle,
-                                                   noverlap=pts_per_ncycle - 1,
-                                                   nfft=nfft,
-                                                   window=self.window,
-                                                   mode='magnitude')
-
-        # Parabolic ridge finder
-        inst_freq, amplitude, _ = parab.ridge_finder(spectrogram, freq)
-
-        # Correctly pad the signals
-        _pts = self.n_points - len(inst_freq)
-        _pre = int(np.floor(_pts / 2))
-        _post = int(np.ceil(_pts / 2))
-
-        inst_freq = np.pad(inst_freq, (_pre, _post))
-        amplitude = np.pad(amplitude, (_pre, _post))
-
-        phase = spg.cumtrapz(inst_freq)
-        phase = np.append(phase, phase[-1])
-        tidx = int(self.tidx * len(inst_freq) / self.n_points)
-
-        self.amplitude = amplitude
-        self.inst_freq_raw = inst_freq
-        self.inst_freq = inst_freq - inst_freq[tidx]
-        self.spectrogram = spectrogram
-        self.stft_freq = freq
-        self.stft_times = times
-
-        # subtract the w*t line (drive frequency line) from phase
-        start = int(0.3 * tidx)
-        end = int(0.7 * tidx)
-        xfit = np.polyfit(np.arange(start, end), phase[start:end], 1)
-        phase -= (xfit[0] * np.arange(len(inst_freq))) + xfit[1]
-
-        self.phase = phase
-
-        return
-
     def plot(self, newplot=True, raw=False):
         """ 
         Quick visualization of best_fit and cut.
@@ -892,6 +960,7 @@ class Pixel:
 
         # Check the drive frequency.
         if self.check_drive:
+            
             self.check_drive_freq()
 
         # DWT Denoise
@@ -907,6 +976,10 @@ class Pixel:
             # Calculate instantenous frequency using sliding FFT
             self.calculate_stft(**self.fft_params)
 
+        elif self.method == 'fv':
+            
+            self.forcevib()
+
         elif self.method == 'hilbert':
             # Hilbert transform method
 
@@ -960,6 +1033,7 @@ class Pixel:
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
+
             self.inst_freq = self.inst_freq * -1
 
         # Find where the minimum is.
@@ -967,10 +1041,12 @@ class Pixel:
 
         # Restore the length due to FIR filter being causal
         if self.method == 'hilbert':
+            
             self.restore_signal()
 
         # If it's a recombination image invert it to find minimum.
         if self.recombination:
+            
             self.inst_freq = self.inst_freq * -1
             self.best_fit = self.best_fit * -1
             self.cut = self.cut * -1
